/***************************************************************************
 *   Copyright (C) 2007 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef GUITONE_H
#define GUITONE_H

#include "MainWindow.h"
#include "Monotone.h"
#include "ApplicationUpdate.h"

#include <QApplication>
#include <QList>
#include <QMap>
#include <QMutex>

class Guitone : public QApplication
{
	Q_OBJECT

public:
    Guitone(int, char**);
    ~Guitone();
    bool init();
    const QList<MainWindow*> windowList() const;
	
    MainWindow * findMainWindow(QObject *);
    Monotone * getMonotoneInstance(QObject *);
    bool loadFromPath(const QString &);
    
    void lock();
    void unlock();
    
signals:
    void windowListChanged();

private slots:
    bool loadWorkspace(const QString &);
    bool loadDatabase(const QString &);
    void windowClosed(MainWindow *);
    void quit();
    
private:
    MainWindow * addWindow();
    void removeWindow(MainWindow *);
    
    bool addMonotoneInstance(MainWindow *);
    bool removeMonotoneInstance(MainWindow *);
    
#ifdef Q_WS_MACX
    bool macEventFilter(EventHandlerCallRef, EventRef);
#endif

	QList<MainWindow *> openWindows;
    QMap<MainWindow *, Monotone *> monotoneInstances;
    QMutex mutex;
    
    ApplicationUpdate * updateDialog;
};

#endif

