/***************************************************************************
 *   Copyright (C) 2006 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include "Branches.h"
#include "Monotone.h"

Branches::Branches(QObject * parent, bool display_tree) 
    : QAbstractItemModel(parent), tree(display_tree)
{
	builder = 0;
	mtnDelegate = new MonotoneDelegate(this);
}

Branches::~Branches()
{
	if(builder)
		delete builder;
    delete mtnDelegate;
}

bool Branches::readBranches()
{
    branches.clear();
    reset();
    
    QStringList cmd;
    cmd << "branches";
  
    return mtnDelegate->triggerCommand(cmd);
}

void Branches::parseOutput()
{
	branches.setHorizontalHeaderItem(0, new QStandardItem(tr("Branches")));
	if(tree)
	{
		QStandardItem *root = new QStandardItem("*");
		root->setData("*", Qt::ToolTipRole);
		branches.appendRow(root);

		builder = new TreeBuilder(root, this);
		builder->addList(AutomateCommand::data);
	}
	else
	{
		QStringList branchList = AutomateCommand::data.split("\n");

		QStringListIterator iterator(branchList);
		while(iterator.hasNext())
		{
			QString it = iterator.next();
			QStandardItem *item = new QStandardItem(it);
			item->setData(it, Qt::ToolTipRole);
			branches.appendRow(item);
		}
	}
    reset();
    
    emit branchesRead();
}

int Branches::columnCount(const QModelIndex &parent) const
{
	QStandardItem * item = branchFromIndex(parent);
	if(item)
		return item->columnCount();
	else
		return branches.columnCount();
}

QVariant Branches::data(const QModelIndex &index, int role) const
{
	QStandardItem * item = branchFromIndex(index);
	if(item)
		return item->data(role);
	else
		return QVariant();
}

//FIXME: Do we need this?

/*Qt::ItemFlags Branches::flags(const QModelIndex &index) const
{   
	return branchFromIndex(index)->flags();
}*/

QVariant Branches::headerData(int section, Qt::Orientation orientation, int role) const
{
	return branches.headerData(section, orientation, role);
}

int Branches::rowCount(const QModelIndex& parent) const
{
	QStandardItem * item = branchFromIndex(parent);
	if(item)
		return item->rowCount();
	else
		return branches.rowCount();
}

QModelIndex Branches::index(int row, int column, const QModelIndex& parent) const
{
    return createIndex(row, column, getBranchItem(row, column, parent));
}

QModelIndex Branches::parent(const QModelIndex& index) const
{	
	QStandardItem * item = branchFromIndex(index);
	if(item)
	{
		QStandardItem * parent = item->parent();
		if(parent)
			return createIndex(parent->row(), parent->column(), parent);
	}

	return QModelIndex();
}

void * Branches::getBranchItem(int row, int column, const QModelIndex& index) const
{
	if(index.isValid())
		return ((QStandardItem *) index.internalPointer())->child(row, column);
	else
		return branches.item(row, column);
}

QStandardItem * Branches::branchFromIndex(const QModelIndex& index) const
{
	if(index.isValid())
		return static_cast<QStandardItem*>(index.internalPointer());
	else
		return 0;
}
