/***************************************************************************
 *   Copyright (C) 2006 by Jean-Louis Fuchs                                 *
 *   ganwell@fangorn.ch                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef CHANGESETMODE_H
#define CHANGESETMODE_H

#include "Changeset.h"
#include "GetBranchLog.h"
#include "Certs.h"

#include <QtCore>
#include <QModelIndex>

struct BranchInfo
{
	QStringList revisions;
	int certsRead;
};

class ChangesetCerts : public Certs
{
    Q_OBJECT
public:
	ChangesetCerts(QString branch, QString revision, int index, QObject *parent) : Certs(parent)
	{
		this->revision = revision;
		this->idx = index;
		this->branch = branch;
		connect(this, SIGNAL(certsRead()),
			this, SLOT(changesetCertsRead()));
		connect(this, SIGNAL(changesetCertReady(ChangesetCerts *)),
			parent, SLOT(changesetCertReady(ChangesetCerts *)));
	}

	~ChangesetCerts() {}

	QString getRevision()
	{
		return revision;
	}

	int getIndex()
	{
		return idx;
	}

	QString getBranch()
	{
		return branch;
	}

signals:
	void changesetCertReady(ChangesetCerts *);

private:
	QString revision;
	int idx;
	QString branch;

private slots:
	void changesetCertsRead()
	{
		emit changesetCertReady(this);
	}
};

typedef QMap<QString, Changeset*> ChangesetMap;
typedef QMap<QString, BranchInfo> BranchMap;

class ChangesetModel : public QAbstractItemModel
{
	Q_OBJECT

public:
	ChangesetMap branchModel;
	ChangesetModel(QObject *parent = 0);
	~ChangesetModel();

	void setBranch(QString branch);		
	void receiveRevisions(bool all);

	// needed Qt Model methods
    QVariant data(const QModelIndex&, int) const;
    Qt::ItemFlags flags(const QModelIndex&) const;
    QVariant headerData(int, Qt::Orientation, int) const;
    QModelIndex index(int, int, const QModelIndex&) const;
    QModelIndex parent(const QModelIndex&) const;
    int rowCount(const QModelIndex&) const;
    int columnCount(const QModelIndex&) const; 

private:
	QString currentBranch;
	static const int revPerReceive = 50;
	BranchMap branchMap;
	ChangesetMap changesetMap;

private slots:
	void branchLogRead(GetBranchLog *);
	void changesetCertReady(ChangesetCerts *);
};

#endif //CHANGESETMODE_H

