/***************************************************************************
 *   Copyright (C) 2006 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
// TODO: enable editing of items as well as dropping/adding them

#ifndef CONTENT_DIFF_H
#define CONTENT_DIFF_H

#include "AutomateCommand.h"
#include "DiffParser.h"
#include "MonotoneDelegate.h"

#include <QString>
#include <QAbstractItemModel>

struct ListLine 
{
    enum Type { FileLine, HunkSep, HunkLine };
    
    Type type;
    QString firstColumn;
    QString secondColumn;
    ListLine * parent;
    QList<ListLine*> lines;
    
    ListLine() : type(HunkLine), parent(0) {}
    ListLine(Type t) : type(t), parent(0)
    {
        if (type == HunkSep) firstColumn = "..."; 
    }
	inline ~ListLine()
	{
		qDeleteAll(lines);
        lines.clear();
	}

    void addChild(ListLine * l) { lines.append(l); l->parent = this; }
};

typedef QList<ListLine*> ListLines;

class ContentDiff : public QAbstractItemModel, public AutomateCommand
{
    Q_OBJECT
public:
    ContentDiff(QObject*);
    virtual ~ContentDiff();
     
    // needed Qt Model methods
    QVariant data(const QModelIndex&, int) const;
    Qt::ItemFlags flags(const QModelIndex&) const;
    QVariant headerData(int, Qt::Orientation, int) const;
    QModelIndex index(int, int, const QModelIndex&) const;
    QModelIndex parent(const QModelIndex&) const;
    int rowCount(const QModelIndex&) const;
    int columnCount(const QModelIndex&) const; 

    inline Diff * getDiff(QString fileName) { return diffParser->getDiff(fileName); }
    inline FileDiffs getAllDiffs() { return diffParser->getAllDiffs(); }
    
public slots:
    bool readDiff(QString);
    bool readDiff(QString, QString);
    bool readDiff(QString, QString, QString);

signals:
    void diffRead();

private:
    void parseOutput();
    DiffParser * diffParser;
    ListLines lines;
    MonotoneDelegate * mtnDelegate;
};

#endif
