/***************************************************************************
 *   Copyright (C) 2006 by Jean-Louis Fuchs                                 *
 *   ganwell@fangorn.ch                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "GetBranchLog.h"

GetBranchLog::GetBranchLog(QString branch, QObject *parent) : QObject(parent)
{
	this->branch = branch;
	selectModel = new Select(this);
	revSortModel = new Toposort(this);
	ancestorModel = new Ancestors(this);
	revSortModel->setSourceModel(ancestorModel);
	connect(selectModel, SIGNAL(selectionRead()),
			this, SLOT(selectionReady()));
	connect(revSortModel, SIGNAL(sortingFinished()),
		this, SLOT(revisionsReady()));
	connect(this, SIGNAL(commandDone(GetBranchLog *)),
		parent, SLOT(branchLogRead(GetBranchLog *)));
	selectModel->readSelection("h:" + branch);
}

GetBranchLog::~GetBranchLog()
{
	delete selectModel;
	delete revSortModel;
	delete ancestorModel;
}

QString GetBranchLog::getBranch()
{
	return branch;
}

QStringList GetBranchLog::getRevisions()
{
	return revisions;
}

void GetBranchLog::selectionReady()
{
	QStringList heads;
	int count = selectModel->rowCount(QModelIndex());
	for(int i = 0; i < count; i++)
	{
		heads << selectModel->data(selectModel->index(i, 0, QModelIndex()), Qt::DisplayRole).toString();
	}
	revisions << heads;
	ancestorModel->readAncestors(heads);
}

void GetBranchLog::revisionsReady()
{
	int count = revSortModel->rowCount(QModelIndex());
	for(int i = 0; i < count; i++)
	{
		revSortModel->sort(0, Qt::DescendingOrder);
		revisions << revSortModel->QSortFilterProxyModel::data(revSortModel->index(i, 0, QModelIndex()), Qt::DisplayRole).toString();
	}
	emit commandDone(this);
}
