/***************************************************************************
 *   Copyright (C) 2006 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#ifndef GET_REVISION_H
#define GET_REVISION_H

#include "AutomateCommand.h"
#include "MonotoneDelegate.h"
#include "BasicIOParser.h"

#include <QAbstractItemModel>

enum ChangeType { Delete, Rename, AddDir, AddFile, Patch, AttrSet, AttrClear };

struct Change {
    ChangeType type;
    Stanza stanza;
    
    inline QString getTypeString()
    {
        switch (type)
        {
            case Delete:    return QObject::tr("Delete");
            case Rename:    return QObject::tr("Rename");
            case AddDir:    return QObject::tr("Dir add");
            case AddFile:   return QObject::tr("File add");
            case Patch:     return QObject::tr("Patch");
            case AttrSet:   return QObject::tr("Attr set");
            case AttrClear: return QObject::tr("Attr clear");
        }
        return QString();
    }
    
    inline Qt::GlobalColor getTypeColor()
    {
        switch (type)
        {
            case Delete:    return Qt::red;
            case Rename:    return Qt::gray;
            case AddDir:    return Qt::darkGreen;
            case AddFile:   return Qt::green;
            case Patch:     return Qt::yellow;
            case AttrSet:   return Qt::cyan;
            case AttrClear: return Qt::magenta;
        }
        return Qt::transparent;
    }
    
    inline QString escape(const QString & in)
    {
        QString out(in);
        out.replace("\\", "\\\\");
        out.replace("\"", "\\\"");
        return out;
    }
    
    inline QString getStanzaData()
    {
        switch (type)
        {
            case AddDir:
                return QString("add_dir \"%1\"")
                    .arg(escape(stanza.at(0).vals.at(0)));
            case AddFile:
                return QString("add_file \"%1\"\ncontent [%2]")
                    .arg(escape(stanza.at(0).vals.at(0)))
                    .arg(stanza.at(1).vals.at(0));
            case Delete:
                return QString("delete \"%1\"")
                    .arg(escape(stanza.at(0).vals.at(0)));
            case Patch:
                return QString("patch \"%1\"\nfrom [%2]\nto [%3]")
                    .arg(escape(stanza.at(0).vals.at(0)))
                    .arg(stanza.at(1).vals.at(0))
                    .arg(stanza.at(2).vals.at(0));
            case Rename:
                return QString("rename \"%1\"\nto \"%2\"")
                    .arg(escape(stanza.at(0).vals.at(0)))
                    .arg(escape(stanza.at(1).vals.at(0)));
            case AttrClear:
                return QString("clear \"%1\"\nattr \"%2\"")
                    .arg(escape(stanza.at(0).vals.at(0)))
                    .arg(escape(stanza.at(1).vals.at(0)));
            case AttrSet:
                return QString("set \"%1\"\nattr \"%2\"\nvalue \"%3\"")
                    .arg(stanza.at(0).vals.at(0))
                    .arg(stanza.at(1).vals.at(0))
                    .arg(stanza.at(2).vals.at(0));
        }
        return QString();
    }
    
    inline QString getDisplayData()
    {
        switch (type)
        {
            case AddDir:
            case AddFile:
            case Patch:
            case Delete:
                return stanza.at(0).vals.at(0);
            case Rename:
                return QObject::tr("%1 to\n%2")
                    .arg(stanza.at(0).vals.at(0))
                    .arg(stanza.at(1).vals.at(0));
            case AttrClear:
                return QObject::tr("'%1' from\n%2")
                    .arg(stanza.at(1).vals.at(0))
                    .arg(stanza.at(0).vals.at(0));
            case AttrSet:
                return QObject::tr("'%1' to\n'%2' for\n%3")
                    .arg(stanza.at(1).vals.at(0))
                    .arg(stanza.at(2).vals.at(0))
                    .arg(stanza.at(0).vals.at(0));
        }
        return QString();
    }
};

struct Revision
{
    QString new_manifest;
    QMap<QString, QList<Change > > changesAgainstParent;
    
    inline void clear()
    {
        new_manifest.clear();
        changesAgainstParent.clear();
    }
};

class GetRevision : public QAbstractItemModel, public AutomateCommand
{
    Q_OBJECT
public:
    GetRevision(QObject *);
    virtual ~GetRevision();
    QStringList getParentRevisions() const;
    
    // needed Qt Model methods
    QVariant data(const QModelIndex &, int) const;
    Qt::ItemFlags flags(const QModelIndex &) const;
    QVariant headerData(int, Qt::Orientation, int) const;
    QModelIndex index(int, int, const QModelIndex &) const;
    QModelIndex parent(const QModelIndex &) const;
    int rowCount(const QModelIndex &) const;
    int columnCount(const QModelIndex &) const; 

public slots:
    bool readRevision(const QString &);
    void showChangesAgainstParent(const QString &);
    
signals:
    void revisionRead();

private:
    void parseOutput();

    MonotoneDelegate * mtnDelegate;
    Revision revision;
    QString changesAgainstParent;
};

#endif
