/***************************************************************************
 *   Copyright (C) 2006 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef INVENTORY_H
#define INVENTORY_H

#include "AutomateCommand.h"
#include "MonotoneDelegate.h"

#include <QRegExp>
#include <QAbstractItemModel>

class InventoryItem;

class Inventory : public QAbstractItemModel, public AutomateCommand
{
    Q_OBJECT

    public:
        Inventory(QObject *parent);
        ~Inventory();
        bool readInventory();
        static QString getOption(QObject *, const QString &);
        static QString getBranchName(QObject *);
        static QString getBranchNameShort(QObject *);
        
        // needed Qt Model methods
        QVariant data(const QModelIndex&, int) const;
        bool setData(const QModelIndex &, const QVariant &, int role = Qt::EditRole);
        bool setItemData(const QModelIndex &, const QMap<int, QVariant> &);
        Qt::ItemFlags flags(const QModelIndex&) const;
        QVariant headerData(int, Qt::Orientation, int) const;
        QModelIndex index(int, int, const QModelIndex&) const;
        QModelIndex parent(const QModelIndex&) const;
        int rowCount(const QModelIndex&) const;
        int columnCount(const QModelIndex&) const;
        
    private:
        void parseOutput();
        bool handleError(int);
		bool parseInventoryLine(const QString &, int &, int &, int &, QString &, bool &);
		QList<InventoryItem*> buildTreeRecursive(QList<InventoryItem*> &, InventoryItem*);
		
        InventoryItem	 * rootItem;
		QRegExp			 * regex;
        MonotoneDelegate * mtnDelegate;
		QString            branchName;
    
    private slots:
        void loadBranchName();
    
    signals:
        void modelCreated();
        void invalidWorkspaceFormat(const QString &);
};

#endif

