/***************************************************************************
 *   Copyright (C) 2006 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef INVENTORY_ITEM_H
#define INVENTORY_ITEM_H

#include <QVariant>

class InventoryItem : public QObject
{
    Q_OBJECT
    
public:
    InventoryItem(bool isDir = false, bool isRoot = false);
    InventoryItem(const InventoryItem *);
    ~InventoryItem(void);
    
    inline void setRenamedFrom(InventoryItem * from) { renamed_from = from; }
    inline void setRenamedTo(InventoryItem * to) { renamed_to = to; }
    inline InventoryItem * getRenamedFrom(void) const { return renamed_from; }
    inline InventoryItem * getRenamedTo(void) const { return renamed_to; }
    
    inline void setParent(InventoryItem * p) { parentItem = p; }
    inline InventoryItem * parent() const { return parentItem; }
    inline QList<InventoryItem*> getChildren(void) const { return children; }
    
    inline QString getPath(void) const { return path; }
    inline void setPath(const QString & p) { path = p; }        
    
    inline bool isCdUp() const { return getFilename() == ".."; }
    inline bool isTracked() const { return hasNotStatus(Ignored | Unknown); }
    inline bool isDirectory(void) const { return dirFlag; }
    inline bool isRootDirectory(void) const { return rootFlag; }
    inline int getStatus(void) const { return status; }
    inline void setStatus(int st) { status = st; }
    inline void setLabel(const QString & l) { label = l; }
    inline QString getLabel() const { return label; }
    
    inline InventoryItem * child(int row) const { return children.value(row); }
    inline int childCount(void) const { return children.count(); };
    inline int columnCount(void) const { return 3; }
    
    void setChildren(QList<InventoryItem*>);
    void deleteAllChildren(void);
    
    QString getRelativePath(const QString&) const;
    QString getFilename(void) const;
    QString getBaseDirectory(void) const;
    
    bool hasStatus(int) const;
    bool hasNotStatus(int) const;
    int getStatusRecursive() const;
    bool hasChangedRecursive() const;
    
    /* needed for Qt model class */
    int row(void) const;
    void appendChild(InventoryItem*);
    QVariant data(int, int) const;
    
    static const int RenamedFrom;
    static const int RenamedTo;
    static const int Added;
    static const int Dropped;
    static const int Missing;
    static const int Patched;
    static const int Unchanged;
    static const int Unknown;
    static const int Ignored;
    
    static const QList<int> ValidStates;

private:
    QString getItemName(void) const;
    QString getStatusString(void) const;
    QString getRenameInfo(void) const;
    
    InventoryItem * renamed_from;
    InventoryItem * renamed_to;
    InventoryItem * parentItem;
    QList<InventoryItem*> children;
    int status;
    bool dirFlag;
    bool rootFlag;
    QString path;
    QString label;
};

#endif
