/***************************************************************************
 *   Copyright (C) 2006 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include "Select.h"
#include "Toposort.h"
#include "Monotone.h"

#include <QFont>

Select::Select(QObject *parent) 
    : QAbstractItemModel(parent)
{
    selRevisions = new RevisionList();
	mtnDelegate = new MonotoneDelegate(this);
}

Select::~Select()
{
    if (selRevisions)
	{
		selRevisions->clear();
		delete selRevisions;
	}

	delete mtnDelegate;
}

bool Select::readSelection(const QString & selector)
{
	// clear current attributes list
    selRevisions->clear();
    // reset the view
    reset();
    
    QStringList cmd;
    cmd.append("select");
    cmd.append(selector);
    
    return mtnDelegate->triggerCommand(cmd);
}

bool Select::handleError(int errCode)
{
    // we can't handle syntax and other basic errors
    if (errCode == 1) return false;
    
    // since there is no other indication we assume that errCode 2
    // is always fired if a invalid selector syntax has been given
    emit invalidSelection(AutomateCommand::data);
     
    return true;
}

void Select::parseOutput()
{
	if (selRevisions > 0)
	{
		selRevisions->clear();
		delete selRevisions;
		selRevisions = new RevisionList(
            AutomateCommand::data.split('\n', QString::SkipEmptyParts)
        );
		// reset the view
		reset();
	}

    // signal that we've finished (whoever listens to that)
    emit selectionRead();
}

int Select::columnCount(const QModelIndex &parent) const
{
	return 1;
}

QVariant Select::data(const QModelIndex &index, int role) const
{
	if (!index.isValid())
	{
		return QVariant();
	}
    
    if (role == Qt::FontRole)
    {
        QFont font;
        font.setStyleHint(QFont::Courier);
        font.setFamily("Courier");
        return QVariant(font);
    }
    
    if (role == Qt::DisplayRole)
	{
        int row = index.row();
        if (row >= selRevisions->size()) return QVariant();
        return QVariant(selRevisions->at(row));
    }
    
	return QVariant();
}

Qt::ItemFlags Select::flags(const QModelIndex &index) const
{
    if (index.isValid())
    {
        return Qt::ItemIsEnabled | Qt::ItemIsSelectable;
    }
    return 0;
}

QVariant Select::headerData(int section, Qt::Orientation orientation, int role) const
{
	if (orientation == Qt::Horizontal && role == Qt::DisplayRole)
	{
        return QVariant(tr("Revision ID"));
    }
    return QVariant();
}

int Select::rowCount(const QModelIndex& parent) const
{
	return selRevisions->size();
}

QModelIndex Select::index(int row, int column, const QModelIndex& parent) const
{
    if (!hasIndex(row, column, parent))
    {
        return QModelIndex();
    }
    
    return createIndex(row, column, 0);
}

QModelIndex Select::parent(const QModelIndex& index) const
{
    return QModelIndex();
}

