/***************************************************************************
 *   Copyright (C) 2006 by Jean-Louis Fuchs                                *
 *   ganwell@fangorn.ch                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include "Toposort.h"
#include "Monotone.h"

#include <QStringList>
#include <QAbstractItemModel>

Toposort::Toposort(QObject *parent) : QSortFilterProxyModel(parent),
  sortColumn(0), sortOrder(Qt::AscendingOrder)
{
    ordRevisions = new RevisionMap();
    mtnDelegate  = new MonotoneDelegate(this);
    showRows     = false;
}

Toposort::~Toposort()
{
	if (ordRevisions)
	{
		ordRevisions->clear();
		delete ordRevisions;
	}
    delete mtnDelegate;
}

void Toposort::setSourceModel(QAbstractItemModel * source)
{
    disconnect(
        sourceModel(), SIGNAL(modelReset()),
        this, SLOT(sortModel())
    );
    
    QSortFilterProxyModel::setSourceModel(source);

    connect(
        source, SIGNAL(modelReset()),
        this, SLOT(sortModel())
    );
}

bool Toposort::lessThan(const QModelIndex &left, const QModelIndex &right) const
{
    QString leftRev = sourceModel()->data(left).toString();
    QString rightRev = sourceModel()->data(right).toString();
    
    if (!ordRevisions->contains(leftRev) || !ordRevisions->contains(leftRev))
    {
        D("One or both revisions not found.");
        return false;
    }
    
    return ordRevisions->value(leftRev) < ordRevisions->value(rightRev);
}

bool Toposort::filterAcceptsRow(int sourceRow, const QModelIndex &sourceParent) const
{
    return showRows;
}

void Toposort::sort(int column, Qt::SortOrder order)
{
    sortOrder = order;
	sortColumn = column;
    QSortFilterProxyModel::sort(column, order);
}

void Toposort::sortModel()
{
    showRows = false;
    ordRevisions->clear();
    
    int row = 0;
    QModelIndex current = sourceModel()->index(row,0);
    if (!current.isValid())
    {
        D("No items found in source model");
        return;
    }
    
    QStringList revs;
    do
    {
        QString rev = current.data().toString();
        // skip everything what does not look like a revision id
        if (rev.size() == 40)
        {
            revs.append(rev);
        }
        current = current.sibling(++row, 0);
        if (!current.isValid()) break;
    }
    while (true);
    
    QStringList cmd;
    cmd.append("toposort");
	cmd << revs;
    
    if (!mtnDelegate->triggerCommand(cmd))
    {
        D("Cannot trigger toposort command");
    }
}

void Toposort::parseOutput()
{   
    QStringList sortedRevs(AutomateCommand::data.split("\n"));
    for (int i=0, j=sortedRevs.size(); i<j; i++)
    {
        ordRevisions->insert(sortedRevs.at(i), i);
    }
    
    showRows = true;
    filterChanged();
    clear();
    
    D("Sorting finished");
    
    // signal that we've finished (whoever listens to that)
    emit sortingFinished();
}

