/***************************************************************************
 *   Copyright (C) 2007 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include "FileExporter.h"
#include "MonotoneDelegate.h"
#include "Guitone.h"

#include <QProgressDialog>

FileExporter::FileExporter(QObject * parent, QString rev, QDir dir)
    : QObject(parent), revision(rev)
{
    rootDir = dir;
    I(rootDir.exists());
}

FileExporter::~FileExporter()
{
}

bool FileExporter::run(const FileEntryList & entries)
{
    QProgressDialog progress(
        tr("Processing files..."), tr("Abort"), 0, entries.size()
    );
    progress.setWindowModality(Qt::WindowModal);

    bool success = true;
    bool cleanup = false;
    for (int i=0, j=entries.size(); i < j; i++)
    {
        if (!exportFile(entries.at(i)))
        {
            success = false;
            cleanup = true;
            break;
        }
        
        APP->processEvents();
        if (progress.wasCanceled())
        {
            cleanup = true;
            break;
        }
        progress.setValue(i + 1);
    }
    
    if (cleanup)
    {
        D("TODO: clean up written files");
    }
    return success;
}

bool FileExporter::exportFile(const FileEntry & entry)
{
    if (entry.second)
    {
        if (!entry.first.isEmpty() && !rootDir.mkpath(entry.first))
        {
            C("Cannot create directory");
            return false;
        }
        return true;
    }
    
    Monotone * mtn = MTN(this);
    
    int commandNumber;
    if (!mtn->executeCommand(
            QStringList() << "get_file_of" << entry.first, 
            QStringList() << "r" << revision, commandNumber) ||
        mtn->getReturnCode(commandNumber) > 0)
    {
        C("Cannot execute get_file_of");
        return false;
    }
    
    QFile file(rootDir.filePath(entry.first));
    if (file.exists()) file.remove();
    
    if (!file.open(QIODevice::WriteOnly))
    {
        C("Can't open file for writing");
        return false;
    }
    
    I(file.write(mtn->getRawData(commandNumber)));
    file.close();
    
    return true;
}

