/***************************************************************************
 *   Copyright (C) 2006 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef MONOTONE_H
#define MONOTONE_H

#include "vocab.h"

#include <QProcess>
#include <QMap>
#include <QByteArray>
#include <QMutex>

class Monotone : public QObject
{
	Q_OBJECT

	public:
        static bool runCommand(const QString &, const QStringList &, QString &);
        static bool checkProgramVersion(const QString &);
        static bool checkInterfaceVersion(const QString &);
        static QString stripMtnPrefix(const QString &);
        static int versionCompare(const QString &, const QString &);
        static ByteArrayList stringToByteArrayList(const QStringList &);
        static const QString RequiredProgramVersion;
        static const QString RequiredInterfaceVersion;
        static const int StdioBufferSize;
        static const int WaitForMonotoneStart;

        Monotone(QObject *);
        virtual ~Monotone();
        
		bool loadWorkspace(QString);
		bool loadDatabase(QString);
        bool setMtnBinaryPath(const QString &);
        QString getNormalizedWorkspacePath() const;
        QString getDatabaseFilePath() const;
        QString getStderr();

        bool triggerCommand(const QStringList &, int &);
        bool triggerCommand(const ByteArrayList &, int &);
        bool triggerCommand(const QStringList &, const QStringList &, int &);
        bool triggerCommand(const ByteArrayList &, const ByteArrayList &, int &);
        
        bool executeCommand(const QStringList &, int &);
        bool executeCommand(const ByteArrayList &, int &);
        bool executeCommand(const QStringList &, const QStringList &, int &);
        bool executeCommand(const ByteArrayList &, const ByteArrayList &, int &);
        
        QString getDecodedData(int);
        QByteArray getRawData(int);
        int getReturnCode(int);
        bool isCommandFinished(int) const;

    private:
        enum Mode { Workspace, Database } mode;
        
        bool normalizeWorkspacePath(QString &) const;
        bool setupNewCommand();
        bool setupNewProcess();
        void shutdownCurrentProcess();
        int writeStdin(const ByteArrayList &, const ByteArrayList &);
        void timerEvent(QTimerEvent *);
        
        QByteArray input;
		QMap<int, QByteArray> output;
        QMap<int, int> completedCommands;
        int commandCounter;
        bool isCleanExit;
        bool doAbortRequests;
		QProcess * process;
        QString workDir;
        QString databaseFile;
        QString mtnBinaryPath;
        QMutex lock;
        
        int timer;
        bool timeout;

	private slots:
		void readAndParseStdout();
		void processFinished(int, QProcess::ExitStatus);
        void processError(QProcess::ProcessError);
        
	signals:
		void commandFinished();
};

#endif
