/***************************************************************************
 *   Copyright (C) 2006 by Ingo Maindorfer								   *
 *   ingo@liquidcooling.de												   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "IconProvider.h"
#include "InventoryItem.h"

#include <QStyle>
#include <QApplication>
#include <QPainter>

const int IconProvider::CdUp = -1;
IconProvider * IconProvider::instance = 0;

IconProvider * IconProvider::singleton()
{
    if (instance == 0)
    {
        instance = new IconProvider();
    }
    return instance;
}

/**
 * There are 45 possible status combinations, of which 19 are invalid.
 * We only provide for the 26 valid ones own icons
 * (more info: http://venge.net/monotone/docs/Automation.html#Automation)
 * 
 * Basic codes
 * -----------
 *
 *          M 0 0 missing
 *         AP 0 0 added
 *        D   0 0 dropped
 *        R   1 0 renamed-from-this
 *         R  0 1 renamed-to-this
 *          P 0 0 patched
 *            0 0 unchanged
 *          U 0 0 unknown
 *          I 0 0 ignored
 *
 * Combinations and used icons
 * ---------------------------
 *
 * In general, we don't show the "patched" state if there is another
 * state up to the item (added, renamed, etc.)
 *
 * '   ': unchanged.png
 * '  P': patched.png
 * '  U': unknown.png
 * '  I': ignored.png
 * '  M': missing.png
 *
 * ' AP': added.png
 * ' AM': added_missing.png
 *
 * ' R ': rename_target.png
 * ' RP': rename_target.png
 * ' RM': rename_target_missing.png
 *
 * 'D  ': dropped.png
 * 'D U': dropped_unknown_ignored.png
 * 'D I': dropped_unknown_ignored.png
 *
 * 'DAP': dropped_added.png
 * 'DAM': dropped_added_missing.png
 *
 * 'DR ': dropped_rename_target.png
 * 'DRP': dropped_rename_target.png
 * 'DRM': dropped_rename_target_missing.png
 *
 * 'R  ': rename_source.png
 * 'R U': rename_source_unknown_ignored.png
 * 'R I': rename_source_unknown_ignored.png
 *
 * 'RAP': rename_source_added.png
 * 'RAM': rename_source_added_missing.png
 *
 * 'RR ': rename_source_target.png
 * 'RRP': rename_source_target.png
 * 'RRM': rename_source_target_missing.png
 */

IconProvider::IconProvider()
{
    QStyle *style = QApplication::style();

    //
    // default icons if no state matches
    //
    fileIcons[0] = QIcon(style->standardPixmap(QStyle::SP_FileIcon));
    folderIcons[0] = QIcon(style->standardPixmap(QStyle::SP_DirClosedIcon));
    folderIcons[0].addPixmap(style->standardPixmap(QStyle::SP_DirOpenIcon), QIcon::Active, QIcon::On); 
    
    //
    // assign possible item states to the appropriate overlay
    //
    QMap<int, QString> states;
    states[InventoryItem::Unchanged] = QString(":/overlays/unchanged.png");
    states[InventoryItem::Patched] = QString(":/overlays/patched.png");
    states[InventoryItem::Unknown] = QString(":/overlays/unknown.png");
    states[InventoryItem::Ignored] = QString(":/overlays/ignored.png");
    states[InventoryItem::Missing] = QString(":/overlays/missing.png");
    
    states[InventoryItem::Added|InventoryItem::Patched] = QString(":/overlays/added.png");
    states[InventoryItem::Added|InventoryItem::Missing] = QString(":/overlays/added_missing.png");
    
    states[InventoryItem::RenamedTo|InventoryItem::Unchanged] = QString(":/overlays/rename_target.png");
    states[InventoryItem::RenamedTo|InventoryItem::Patched] = QString(":/overlays/rename_target.png");
    states[InventoryItem::RenamedTo|InventoryItem::Missing] = QString(":/overlays/rename_target_missing.png");
    
    states[InventoryItem::Dropped|InventoryItem::Unchanged] = QString(":/overlays/dropped.png");
    states[InventoryItem::Dropped|InventoryItem::Unknown] = QString(":/overlays/dropped_unknown_ignored.png");
    states[InventoryItem::Dropped|InventoryItem::Ignored] = QString(":/overlays/dropped_unknown_ignored.png");
    
    states[InventoryItem::Dropped|InventoryItem::Added|InventoryItem::Patched] = QString(":/overlays/dropped_added.png");
    states[InventoryItem::Dropped|InventoryItem::Added|InventoryItem::Missing] = QString(":/overlays/dropped_added_missing.png");

    states[InventoryItem::Dropped|InventoryItem::RenamedTo|InventoryItem::Unchanged] = QString(":/overlays/dropped_rename_target.png");
    states[InventoryItem::Dropped|InventoryItem::RenamedTo|InventoryItem::Patched] = QString(":/overlays/dropped_rename_target.png");
    states[InventoryItem::Dropped|InventoryItem::RenamedTo|InventoryItem::Missing] = QString(":/overlays/dropped_rename_target_missing.png");

    states[InventoryItem::RenamedFrom|InventoryItem::Unchanged] = QString(":/overlays/rename_source.png");
    states[InventoryItem::RenamedFrom|InventoryItem::Unknown] = QString(":/overlays/rename_source_unknown_ignored.png");
    states[InventoryItem::RenamedFrom|InventoryItem::Ignored] = QString(":/overlays/rename_source_unknown_ignored.png");

    states[InventoryItem::RenamedFrom|InventoryItem::Added|InventoryItem::Patched] = QString(":/overlays/rename_source_added.png");
    states[InventoryItem::RenamedFrom|InventoryItem::Added|InventoryItem::Missing] = QString(":/overlays/rename_source_added_missing.png");
    
    states[InventoryItem::RenamedFrom|InventoryItem::RenamedTo|InventoryItem::Unchanged] = QString(":/overlays/rename_source_target.png");
    states[InventoryItem::RenamedFrom|InventoryItem::RenamedTo|InventoryItem::Patched] = QString(":/overlays/rename_source_target.png");
    states[InventoryItem::RenamedFrom|InventoryItem::RenamedTo|InventoryItem::Missing] = QString(":/overlays/rename_source_target_missing.png");
    
    // some special icon overlays
    states[IconProvider::CdUp] = QString(":/overlays/cdup.png");
    
    QMapIterator<int, QString> i(states);
    
    while (i.hasNext())
    {
        i.next();
        int state = i.key();
        QImage overlay(i.value());
        overlay.createAlphaMask();

        // the basic icons have to be recreated for each state
        // since we later paint on them
        QImage fileImage(style->standardPixmap(QStyle::SP_FileIcon).toImage());
        QImage folderImageClosed(style->standardPixmap(QStyle::SP_DirClosedIcon).toImage());
        QImage folderImageOpened(style->standardPixmap(QStyle::SP_DirOpenIcon).toImage());
        
        // note: the overlays are scaled to fit the different icon sizes 
        // on different platforms / with different stylings, so the overlay
        // always perfectly fits the actual image
        
        // file icon
        QPainter filePainter(&fileImage);
        filePainter.drawImage(
            0, 0, 
            overlay.scaled(fileImage.width(), fileImage.height())
        );
        filePainter.end();
        fileIcons[state] = QIcon(QPixmap::fromImage(fileImage));
        
        // folder icons
        QPainter folderPainter1(&folderImageClosed);
        folderPainter1.drawImage(
            0, 0,
            overlay.scaled(folderImageClosed.width(), folderImageClosed.height())
        );
        folderPainter1.end();
        QPainter folderPainter2(&folderImageOpened);
        folderPainter2.drawImage(
            0, 0,
            overlay.scaled(folderImageOpened.width(), folderImageOpened.height())
        );
        folderPainter2.end();
    
        folderIcons[state] = QIcon(QPixmap::fromImage(folderImageClosed));
        folderIcons[state].addPixmap(QPixmap::fromImage(folderImageOpened), QIcon::Active, QIcon::On);        
    }
}

IconProvider::~IconProvider() {}

QIcon IconProvider::getIcon(InventoryItem * item) const
{
    if (item->isCdUp())
    {
        return folderIcons.value(IconProvider::CdUp);
    }
    
    int status = item->getStatus();

    if (item->isDirectory())
	{
        if (folderIcons.contains(status))
        {
            return folderIcons.value(status);
        }
        // default icon
        return folderIcons.value(0);
    }
    
    if (fileIcons.contains(status))
    {
        return fileIcons.value(status);
    }
    // default icon
    return fileIcons.value(0);
}


QIcon IconProvider::getPlainFileIcon() const
{
    return fileIcons.value(0);
}

QIcon IconProvider::getPlainFolderIcon() const
{
    return folderIcons.value(0);
}

