/***************************************************************************
*   Copyright (C) 2007 by Thomas Keller                                   *
*   me@thomaskeller.biz                                                   *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include "Platform.h"
#include "vocab.h"

#include <QApplication>

#ifdef Q_WS_X11
#include <cstdlib>
#endif

#ifdef Q_WS_WIN
#include <windows.h>
#include <lmcons.h>
#endif

#ifdef Q_WS_MACX
#include <cstdlib>
#include <qmessagebox.h>
#endif

#include <string>

using namespace std;

//
// The following code is public domain, with slight modifications, taken from
// http://hrabia.ibib.waw.pl/~winnie/qtopen.html
// It opens a file with the associated program on the specific platform
// Original author: Chris Thompson <chris@hypocrite.org>
//
bool Platform::openFile(QWidget *widget, const QString &filename)
{
  bool retval = false;

  QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));

#ifdef Q_WS_X11
  Q_UNUSED(widget);
  // due to a bug in KDE < 3.5.3 we can't trust the return value kfmclient
  // gives us back (its always 1!)
  if (system("which kfmclient >/dev/null 2>&1") == 0)
  {
      D("Using kfmclient");
      string s("kfmclient exec \"");
      s += filename.toUtf8().data();
      s += "\"";
      // just cross your fingers
      system(s.c_str());
      retval = true;
  }
  else
  if (system("which gnome-open >/dev/null 2>&1") == 0)
  {
      D("Using gnome-open");
      string s("gnome-open \"");
      s += filename.toUtf8().data();
      s += "\"";
      retval = (system(s.c_str()) == 0);
  }
  else
  {
     C("Neither kfmclient nor gnome-open found");
  }
#endif

#ifdef Q_WS_MACX
  // Running on a Mac in OS X
  Q_UNUSED(widget);
  string s("open \"");
  s += filename.toUtf8().data();
  s += "\"";
  retval = (system(s.c_str()) == 0);
#endif

#ifdef Q_WS_WIN
  // Running in an MS Windows environment
  if (NULL != widget)
  {
     QString winfilename(filename);
     winfilename.replace(QChar('/'), QString("\\"));
     retval = (reinterpret_cast<int>(ShellExecute(widget->winId(), L"open", 
               winfilename.toStdWString().data(), NULL, NULL, SW_SHOW)) > 32);
  }
#endif

  QApplication::restoreOverrideCursor();
  return retval;
}

//
// the following code is loosely based on David Wright's code snippet
// on the Qt interest mailing list
// (http://lists.trolltech.com/qt-interest/2002-12/thread00130-0.html)
//
QString Platform::getUsername()
{
    QString username;
    
#ifdef Q_WS_WIN
#if defined(UNICODE)
	if ( QSysInfo::WindowsVersion & QSysInfo::WV_NT_based)
    {
        TCHAR winUserName[UNLEN + 1]; // UNLEN is defined in LMCONS.H
        DWORD winUserNameSize = sizeof(winUserName);
        GetUserNameW( winUserName, &winUserNameSize );
		username = QString::fromStdWString( winUserName );
    } else
#endif
    {
        char winUserName[UNLEN + 1]; // UNLEN is defined in LMCONS.H
        DWORD winUserNameSize = sizeof(winUserName);
        GetUserNameA( winUserName, &winUserNameSize );
        username = QString::fromLocal8Bit( winUserName );
    }
#endif

#ifdef Q_WS_X11
#define IS_UNIX
#endif

#ifdef Q_WS_MACX
#define IS_UNIX
#endif

#ifdef IS_UNIX
    username = QString::fromAscii(getenv("USER"));
#endif

    D(QString("Found username %1").arg(username));
    
    return username;
}

