/***************************************************************************
 *   Copyright (C) 2006 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef INVENTORY_VIEW_H
#define INVENTORY_VIEW_H

#include "TreeView.h"
#include "InventoryViewDelegate.h"

#include <QAction>
#include <QSortFilterProxyModel>

class InventoryItem;

class InventoryView : public TreeView
{
	Q_OBJECT

public:
	enum Type { FolderTree, FileList };
	
	InventoryView(QWidget*);
	~InventoryView();

	void setModel(QSortFilterProxyModel *);
    void setType(Type);
    
signals:
    void directoryChanged(const QModelIndex &);

private:
    void setModel(QAbstractItemModel *);
    void createAndConnectContextActions(void);
	void closeEvent(void);
    QModelIndex getSingleSelection(bool mapToSource = true);

    QAction * actChdir;
    QAction * actOpen;
	QAction * actAdd;
	QAction * actRemove;
	QAction * actCommit;
	QAction * actIgnore;
    QAction * actUnignore;
    QAction * actRevert;
    QAction * actRename;
    QAction * actFileDiff;
    QAction * actRevisionDiff;
    
    QAction * actAddMultiple;
    QAction * actRemoveMultiple;
    QAction * actCommitMultiple;
    QAction * actIgnoreMultiple;
    QAction * actUnignoreMultiple;
    QAction * actRevertMultiple;
    
    Type type;
    
    InventoryViewDelegate invViewDelegate;
    
private slots:
    void delegateModelReset();
    void modelReset();
    
    void changeDirectory(const QModelIndex &);
    void itemClicked(const QModelIndex & index);
    void slotContextMenuRequested(const QModelIndexList &, const QPoint &);
    
    void slotChdir(void);
    void slotOpen(void);
	void slotAdd(void);
	void slotRemove(void);
	void slotCommit(void);
	void slotIgnore(void);
    void slotUnignore(void);
    void slotRevert(void);
    void slotRename(void);
    void slotFileDiff(void);
    void slotRevisionDiff(void);
};

#endif
