/***************************************************************************
 *   Copyright (C) 2006 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include "ui_main_window.h"

class QModelIndex;

class Inventory;
class Attributes;
class InventoryProxyModel;
class InventoryView;
class AttributesView;

class MainWindow: public QMainWindow, private Ui::MainWindow
{
    Q_OBJECT

public:
    enum Mode { Workspace = 0, Database};
    MainWindow();
    ~MainWindow();
    
    bool doLoadWorkspace(QString);
    bool doLoadDatabase(QString);
    void enableClosing();
    void disableClosing();
    
public slots:
	void doUpdatePreviousWorkspacesMenu();
    void doUpdatePreviousDatabasesMenu();
    
signals:
    void modeChanged(Mode);
    void windowClosed(MainWindow *);
    void quitApplication();
    void updatePreviousWorkspacesMenu();
    void updatePreviousDatabasesMenu();
	void loadDatabase(const QString &);
	void loadWorkspace(const QString &);
	
private slots:
    void on_actionOpen_Workspace_triggered();
    void on_actionOpen_Database_triggered();
    void on_actionHide_ignored_files_triggered();
    void on_menuShow_triggered(QAction *);
    void on_actionExpand_tree_triggered();
    void on_actionUpdate_workspace_triggered();
    void on_actionCommit_revision_triggered();
    void on_actionCheckout_revision_triggered();
    void on_actionKey_management_triggered();
    void on_actionPreferences_triggered();
    void on_actionAbout_guitone_triggered();
    void on_actionAbout_Qt_triggered();
    void on_actionChangeset_browser_triggered();
    void on_actionBring_all_to_front_triggered();
    void on_actionCheck_for_updates_triggered();
    
    void openRecentWorkspace();
    void openRecentDatabase();
    void updateWindowList();
    void activateOtherWindow();
    void invalidWorkspaceFormat(const QString &);
    
private:
    void closeEvent(QCloseEvent *);
    void switchMode(Mode);
    
    Inventory *invModel;
    Attributes *attrModel;
    InventoryProxyModel *proxyModelFolderTree;
    InventoryProxyModel *proxyModelFileList;
    Mode mode;
    int closeCounter;
};


#endif
