/***************************************************************************
 *   Copyright (C) 2006 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "Splitter.h"
#include "Settings.h"

#include <QApplication>

Splitter::Splitter(QWidget* parent)
: QSplitter(parent)
{
}

Splitter::Splitter(QWidget* parent, QString objName)
: QSplitter(parent)
{
	setObjectName(objName);
}

Splitter::~Splitter() 
{
    saveState();
}

void Splitter::init()
{
	restoreState();
}

void Splitter::restoreState(void)
{
    QSplitter::restoreState(Settings::getSplitterState(objectName()));
}

void Splitter::saveState(void)
{
    // due to a bug in Qt 4.2.2 saveState saves an errornous state for
    // splitters which have not yet been shown to the user. we're avoiding
    // this problem by checking if the sizes() list sums up to 0 which
    // then means that we have just encountered this state
    QList<int> s(sizes());
    int sum = 0;
    for (int i=0, j=s.size(); i<j; ++i)
    {
        sum += s.at(i);
    }
    if (sum == 0) return;
    
    Settings::setSplitterState(QSplitter::saveState(), objectName());
}

