/***************************************************************************
 *   Copyright (C) 2006 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "TreeView.h"
#include "Settings.h"

#include <QHeaderView>

TreeView::TreeView(QWidget* parent)
: QTreeView(parent)
{
}

TreeView::TreeView(QWidget* parent, QString objName)
: QTreeView(parent)
{
	setObjectName(objName);
    init();
}

TreeView::~TreeView() 
{
	saveHeaderViewState();
}

void TreeView::init()
{

    header()->setResizeMode(QHeaderView::Interactive);
    
    stateLoaded = false;   
}

void TreeView::setModel(QAbstractItemModel * model)
{
    QTreeView::setModel(model);
    Settings::restoreHeaderViewState(header(), QString(objectName()).append("_header"));
    stateLoaded = true;
}

void TreeView::saveHeaderViewState()
{
    if (!stateLoaded) return;
    Settings::saveHeaderViewState(header(), QString(objectName()).append("_header"));
}

void TreeView::contextMenuEvent(QContextMenuEvent* ev)
{
    QItemSelectionModel* selection = selectionModel();
    
    // make sure the element under the cursor is selected as well
    QModelIndex currentItem = indexAt(ev->pos());
    selection->select(currentItem, 
        QItemSelectionModel::Select |
        QItemSelectionModel::Rows
    );
    setSelectionModel(selection);
    
    // remove multiple cols of the same column, since we're selected
    // complete rows we're not interested in these
    QModelIndexList currentSelection = selection->selectedIndexes();
    for (int i=0; i < currentSelection.size(); i++)
    {
        if (currentSelection.at(i).column() > 0)
        {
            currentSelection.removeAt(i);
            i--;
        }
    }
    
    emit contextMenuRequested(currentSelection, mapToGlobal(ev->pos()));
}

