/***************************************************************************
 *   Copyright (C) 2007 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "Dialog.h"
#include "Settings.h"
#include "SignalWaiter.h"
#include "MainWindow.h"
#include "vocab.h"

#include <QMenuBar>

Dialog::Dialog(QWidget * parent) : QDialog(parent)
{
    setAttribute(Qt::WA_MacMetalStyle);
}

Dialog::Dialog(QWidget * parent, QString objName)
: QDialog(parent)
{
	setObjectName(objName);
    setAttribute(Qt::WA_MacMetalStyle);
}

Dialog::~Dialog() 
{
	saveState();
}

void Dialog::init()
{
	restoreState();
}

void Dialog::restoreState(void)
{
    restoreGeometry(Settings::getWindowGeometry(objectName()));
}

void Dialog::saveState(void)
{
    Settings::setWindowGeometry(saveGeometry(), objectName());
}

// On OSX this will allow the dialog to be display document modal,
// i.e. as a sheet coming out of the parent window. On other platforms
// this just emulates the behaviour of exec() by blocking until the
// finished() signal has been called
// For more info about document modal dialogs on OSX, go here:
// http://doc.trolltech.com/qq/qq18-macfeatures.html
int Dialog::execDocumentModal()
{
    // ensure that this is only called from top level windows
    QWidget * parent = parentWidget();
    I(parent && parent->inherits("QMainWindow"));
    MainWindow * wnd = qobject_cast<MainWindow*>(parent);
    I(wnd);
    
    // disable the close button's functionality in the MainWindow
    wnd->disableClosing();
    
    QMap<QAction*,bool> enableState;
    
    // disable all menu actions (for some reason disabling the menuBar itself
    // does not result in the desired effect under OSX)
    QList<QAction*> list = wnd->menuBar()->actions();
    foreach(QAction * act, list)
    {
        QList<QAction*> innerList = act->menu()->actions();
        foreach(QAction * innerAct, innerList)
        {
            // skip actions which are later placed into the application menu
            // on OSX or serve other special purposes
            if (innerAct->menuRole() != QAction::NoRole &&
                innerAct->menuRole() != QAction::TextHeuristicRole)
                continue;

            enableState.insert(innerAct, innerAct->isEnabled());
            innerAct->setEnabled(false);
        }
    }
    
    show();
    SignalWaiter waiter(this, SIGNAL(finished(int)));
    waiter.wait();
    
    // restore the enable state for all items
    foreach(QAction * act, enableState.keys())
    {
        act->setEnabled(enableState.value(act));
    }
    
    // enable the close button's functionality in the MainWindow again
    wnd->enableClosing();
    
    return result();
}

void Dialog::setStayOnTop()
{
    if (isVisible()) return;
    setWindowFlags(windowFlags() | Qt::WindowStaysOnTopHint);
}

