/***************************************************************************
 *   Copyright (C) 2006 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "FileDiff.h"
#include "Monotone.h"
#include "SignalWaiter.h"
#include "Settings.h"

#include <QHeaderView>

FileDiff::FileDiff(QWidget* parent) : Dialog(parent) 
{
    setupUi(this);
    Dialog::init();
}

FileDiff::FileDiff(QWidget* parent, QString fileName) : Dialog(parent)
{
    setupUi(this);
    Dialog::init();
    init(fileName);    
}

void FileDiff::init(QString fileName)
{
	connect(
        showVersionLeft, SIGNAL(toggled(bool)),
        this, SLOT(versionToggled(bool))
    );
    connect(
        showVersionRight, SIGNAL(toggled(bool)),
        this, SLOT(versionToggled(bool))
    );
    connect(
        showVersionBoth, SIGNAL(toggled(bool)),
        this, SLOT(versionToggled(bool))
    );
    
    QString title = windowTitle();
    setWindowTitle(title.arg(fileName));
    
    fileModel = new GetFile(this);
    fileModel->readFileByName(fileName);
    
    fileProxyModel = new GetFileProxyModel(this);
    fileProxyModel->setSourceModel(fileModel);

    diffView->setModel(fileProxyModel);
    diffStatusView->setModel(fileProxyModel);
    
    // make the line number col a little smaller
    diffView->header()->resizeSection(0, 40);
    
    SignalWaiter fileWaiter(fileModel, SIGNAL(fileRead()));
    
    if (fileWaiter.wait())
    {
        diffModel = new ContentDiff(this);
        diffModel->readDiff(fileName);
        
        SignalWaiter diffWaiter(diffModel, SIGNAL(diffRead()));
        
        if (diffWaiter.wait())
        {
            fileModel->applyDiff(diffModel->getDiff(fileName));
            diffStatusView->update();
        }
        else
        {
            C("diffWaiter timed out");
        }
    }
    else
    {
        C("fileWaiter timed out");
    }
}

FileDiff::~FileDiff() {}

void FileDiff::versionToggled(bool dummy)
{
    if (showVersionLeft->isChecked())
        fileProxyModel->setFileVersion(GetFileProxyModel::Left);
    if (showVersionRight->isChecked())
        fileProxyModel->setFileVersion(GetFileProxyModel::Right);
    if (showVersionBoth->isChecked())
        fileProxyModel->setFileVersion(GetFileProxyModel::Both);
    
    diffStatusView->update();
}

