/***************************************************************************
 *   Copyright (C) 2006 by Ingo Maindorfer                                 *
 *   ingo@liquidcooling.de                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "GenerateKeypair.h"
#include "Guitone.h"

#include <QMessageBox>
#include <QStringList>

GenerateKeypair::GenerateKeypair(QWidget* parent)
    : Dialog(parent)
{
    setupUi(this);
    Dialog::init();
    
    // OSX sheet-alike dialog
    setWindowFlags(Qt::Sheet);
}

GenerateKeypair::~GenerateKeypair() {}

void GenerateKeypair::accept()
{
    if (lineKeyName->text().size() == 0 ||
        lineKeyPasswd->text().size() == 0 ||
        lineKeyPasswd2->text().size() == 0)
    {
        QMessageBox::information(
            this,
            tr("Missing information"),
            tr("Please fill out all fields."),
            QMessageBox::Ok
        );
        return;
    }
    
    if (lineKeyPasswd->text().compare(lineKeyPasswd2->text()) != 0)
    {
        QMessageBox::information(
            this,
            tr("Password mismatch"),
            tr("The entered passwords did not match."),
            QMessageBox::Ok
        );
        return;
    }
    
    int commandNumber;
    QStringList cmd;
    cmd << "genkey" << lineKeyName->text() << lineKeyPasswd->text();
    
    Monotone * mtn = MTN(this);
    
    if (!mtn->executeCommand(cmd, commandNumber))
    {
       QMessageBox::critical(
             this,
             tr("Unable to execute command"),
			 tr("Unable to execute '%1' - maybe another command is still running?").arg("genkey"),
             QMessageBox::Ok, 0, 0
        );
        return;
    }
    
    QString msg = mtn->getDecodedData(commandNumber);
    
    if (mtn->getReturnCode(commandNumber) > 0)
    {
        QMessageBox::critical(
             this,
             tr("Error creating keypair"),
			 tr("There was an error creating the keypair:\n%1").arg(msg),
             QMessageBox::Ok, 0, 0
        );
        return;
    }
    
    done(QDialog::Accepted);
}

