/***************************************************************************
 *   Copyright (C) 2006 by Ingo Maindorfer                                 *
 *   ingo@liquidcooling.de                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "KeyManagement.h"
#include "GenerateKeypair.h"
#include "Monotone.h"

#include <QMessageBox>
#include <QApplication>
#include <QClipboard>
#include <QTimer>

KeyManagement::KeyManagement(QWidget* parent)
    : Dialog(parent)
{
    setupUi(this);
	Dialog::init();
    
    // OSX sheet-alike dialog
    setWindowFlags(Qt::Sheet);
    
    model = new Keys(this);
    keyList->setModel(model);
    
    if (!model->readKeys())
    {
         QMessageBox::critical(
            this,
            tr("Unable to execute command"),
            tr("Unable to execute '%1' - maybe another command is still running?").arg("keys"),
            QMessageBox::Ok
        );
        QTimer::singleShot(0, this, SLOT(reject()));
        return;
    }
    
    connect(
        generateKey, SIGNAL(clicked()),
        this, SLOT(generateKeypair())
    );
    
    popupMenu = new QMenu(this);
    
    QAction * act = new QAction(tr("Copy key name to clipboard"), this); 
	connect(act, SIGNAL(triggered()), this, SLOT(copyKeyNameToClipboard()));
    popupMenu->addAction(act);
    
    act = new QAction(tr("Copy public key hash to clipboard"), this); 
	connect(act, SIGNAL(triggered()), this, SLOT(copyPubkeyHashToClipboard()));
    popupMenu->addAction(act);
    
    act = new QAction(tr("Copy public key data to clipboard"), this); 
	connect(act, SIGNAL(triggered()), this, SLOT(copyPubkeyDataToClipboard()));
    popupMenu->addAction(act);
}

KeyManagement::~KeyManagement()
{
    delete model;
    qDeleteAll(popupMenu->findChildren<QAction*>());
    delete popupMenu;
}

// TODO: maybe we want to make the keypair generating dialog a small expanding
// widget inside the main dialog?
void KeyManagement::generateKeypair()
{
    // we need to hide the current dialog otherwise we can't show the next
    // dialog as sheet on OSX (its apparently not possible to have more than
    // one sheet opened for a main window at a time)
    hide();
    
    // the sheet must be rooted to the main window otherwise
    // a plain dialog is displayed
    GenerateKeypair dlg(parentWidget());
    
    // if the key was successfully created, reload the key list
    if (dlg.execDocumentModal() == QDialog::Accepted)
    {
        model->readKeys();
    }
    
    // show the keys dialog again
    show();
}

void KeyManagement::contextMenuEvent(QContextMenuEvent* ev)
{
    QItemSelectionModel* selection = keyList->selectionModel();
    QList<QModelIndex> indexList = selection->selectedIndexes();
    if (indexList.size() == 0) return;
    
    popupMenu->exec(mapToGlobal(ev->pos()));
}

Key * KeyManagement::getKeyFromSelection() const
{
    QItemSelectionModel* selection = keyList->selectionModel();
    QList<QModelIndex> indexList = selection->selectedIndexes();
    // we're only interested in the first column's internal pointer
    return static_cast<Key*>(indexList[0].internalPointer());
}

void KeyManagement::copyKeyNameToClipboard()
{
    Key * key = getKeyFromSelection();
    QClipboard * clipboard = QApplication::clipboard();
    clipboard->setText(key->name);
}

void KeyManagement::copyPubkeyHashToClipboard()
{
    Key * key = getKeyFromSelection();
    QClipboard * clipboard = QApplication::clipboard();
    clipboard->setText(key->public_hash);
}

void KeyManagement::copyPubkeyDataToClipboard()
{
    // FIXME: no automate pubkey available yet
    C("Not implemented");
}

